﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace HeuristicLab.Algorithms.Bandits.BanditPolicies {
  // stores information that is relevant for most of the policies
  public class DefaultPolicyActionInfo : IBanditPolicyActionInfo {
    private double knownValue;
    public bool Disabled { get { return Tries == -1; } }
    public double SumReward { get; private set; }
    public int Tries { get; private set; }
    public double MaxReward { get; private set; }
    public double Value {
      get {
        if (Disabled) return knownValue;
        else
          return Tries > 0 ? SumReward / Tries : 0.0;
      }
    }
    public DefaultPolicyActionInfo() {
      MaxReward = double.MinValue;
    }

    public void UpdateReward(double reward) {
      Debug.Assert(!Disabled);

      Tries++;
      SumReward += reward;
      MaxReward = Math.Max(MaxReward, reward);
    }
    public void Disable(double reward) {
      this.Tries = -1;
      this.SumReward = 0.0;
      this.knownValue = reward;
    }
    public void Reset() {
      SumReward = 0.0;
      Tries = 0;
      MaxReward = 0.0;
      knownValue = 0.0;
    }
    public void PrintStats() {
      Console.WriteLine("avg reward {0,5:F2} disabled {1}", SumReward / Tries, Disabled);
    }

    public static Func<DefaultPolicyActionInfo, double> AverageReward {
      get {
        return (aInfo) =>
          aInfo.Tries == 0 ?
          double.PositiveInfinity :
          aInfo.SumReward / (double)aInfo.Tries;
      }
    }
  }
}
